#' adjust center + scale for google map plotting
#'
#' @param x \code{sf} object
#' @param map \code{ggmap} object
#' @importFrom sf st_transform st_coordinates st_crs
#'
#' @return two-column matrix of locations from \code{x} projected to match \code{map}
googlemap_proj <- function(x, map){
  scale_min <- get_googlemap_min_scale(map)
  x_wm <- st_transform(x, crs = st_crs("EPSG:3857"))
  x_gm <- t((t(st_coordinates(x_wm)) - scale_min$min) * scale_min$scale)
  return(x_gm)
}

#' Figure out scale and centering of google map by transforming reported lat long bounding box back to web mercator
#'
#' @param map \code{ggmap} object
#' @importFrom sf st_as_sf st_transform st_coordinates st_crs
#'
#' @return scale (factor by which web mercator has been shrunk) and min (leftmost, bottom most coordinate of rectangle)
get_googlemap_min_scale <- function(map){
  crs_merc <- st_crs(3857)
  crs_ll <- st_crs(4326)
  ## get bb in long/lat from google map object  
  bb_map <- as.numeric(attr(map, "bb"))
  ## make sf obj and transform to match web mercator    
  bb_sf <- st_as_sf(as.data.frame(t(matrix(bb_map, 2, 2))), coords = c("V2", "V1"), 
                    crs = crs_ll)
  bb_wm <- st_transform(bb_sf, crs = crs_merc)
  bb <- st_coordinates(bb_wm)
  min <- bb[1, ]
  scale <- rev(dim(map)) / apply(bb, 2, diff)
  return(list(min = min, scale = scale))
}